@@ -41,8 +41,8 @@
  */
 static char* EditorHelp1 = N_("\
 ~~		insert a line beginning with a single ~\n\
-~b users	add users to the Bcc: field\n\
-~c users	add users to the Cc: field\n\
+~b addresses	add addresses to the Bcc: field\n\
+~c addresses	add addresses to the Cc: field\n\
 ~f messages	include messages\n\
 ~F messages	same as ~f, except also include headers\n\
 ~h		edit the message header\n\
@@ -78,7 +78,7 @@ be_snarf_data (FILE *f, char **buf, int *bufmax, int *buflen, LOFF_T offset,
     tmplen = sizeof (tmp) - tmplen;
   }
 
-  fseeko (f, offset, 0);
+  fseeko (f, offset, SEEK_SET);
   while (bytes > 0)
   {
     if (fgets (p, tmplen - 1, f) == NULL) break;
