@@ -34,6 +34,7 @@
 #endif
 
 #include "mutt_crypt.h"
+#include "mutt_random.h"
 
 #include <string.h>
 #include <ctype.h>
@@ -77,7 +78,7 @@ void mutt_adv_mktemp (BUFFER *buf)
   {
     prefix = mutt_buffer_pool_get ();
     mutt_buffer_strcpy (prefix, buf->data);
-    mutt_sanitize_filename (prefix->data, 1);
+    mutt_sanitize_filename (prefix->data, MUTT_SANITIZE_ALLOW_8BIT);
     mutt_buffer_printf (buf, "%s/%s", NONULL (Tempdir), mutt_b2s (prefix));
     if (lstat (mutt_b2s (buf), &sb) == -1 && errno == ENOENT)
       goto out;
@@ -116,7 +117,7 @@ int mutt_copy_body (FILE *fp, BODY **tgt, BODY *src)
   }
 
   mutt_adv_mktemp (tmp);
-  if (mutt_save_attachment (fp, src, mutt_b2s (tmp), 0, NULL) == -1)
+  if (mutt_save_attachment (fp, src, mutt_b2s (tmp), 0, NULL, 0) == -1)
   {
     mutt_buffer_pool_release (&tmp);
     return -1;
@@ -402,6 +403,7 @@ int mutt_matches_ignore (const char *s, LIST *t)
   return 0;
 }
 
+
 /* Splits src into parts delimited by delimiter.
  * Invokes mapfunc on each part and joins the result back into src.
  * Note this function currently does not preserve trailing delimiters.
@@ -455,32 +457,23 @@ void mutt_buffer_expand_multi_path (BUFFER *src, const char *delimiter)
   delimited_buffer_map_join (src, delimiter, mutt_buffer_expand_path);
 }
 
-char *mutt_expand_path (char *s, size_t slen)
+void mutt_buffer_expand_multi_path_norel (BUFFER *src, const char *delimiter)
 {
-  return _mutt_expand_path (s, slen, 0);
-}
-
-char *_mutt_expand_path (char *s, size_t slen, int rx)
-{
-  BUFFER *s_buf;
-
-  s_buf = mutt_buffer_pool_get ();
-
-  mutt_buffer_addstr (s_buf, NONULL (s));
-  _mutt_buffer_expand_path (s_buf, rx);
-  strfcpy (s, mutt_b2s (s_buf), slen);
-
-  mutt_buffer_pool_release (&s_buf);
-
-  return s;
+  delimited_buffer_map_join (src, delimiter, mutt_buffer_expand_path_norel);
 }
 
 void mutt_buffer_expand_path (BUFFER *src)
 {
-  _mutt_buffer_expand_path (src, 0);
+  _mutt_buffer_expand_path (src, 0, 1);
 }
 
-void _mutt_buffer_expand_path (BUFFER *src, int rx)
+/* Does expansion without relative path expansion */
+void mutt_buffer_expand_path_norel (BUFFER *src)
+{
+  _mutt_buffer_expand_path (src, 0, 0);
+}
+
+void _mutt_buffer_expand_path (BUFFER *src, int rx, int expand_relative)
 {
   BUFFER *p, *q, *tmp;
   const char *s, *tail = "";
@@ -647,14 +640,44 @@ void _mutt_buffer_expand_path (BUFFER *src, int rx)
 
   mutt_buffer_pool_release (&p);
   mutt_buffer_pool_release (&q);
-  mutt_buffer_pool_release (&tmp);
 
 #ifdef USE_IMAP
   /* Rewrite IMAP path in canonical form - aids in string comparisons of
    * folders. May possibly fail, in which case s should be the same. */
   if (mx_is_imap (mutt_b2s (src)))
     imap_expand_path (src);
+  else
 #endif
+    if (expand_relative &&
+        (url_check_scheme (mutt_b2s (src)) == U_UNKNOWN) &&
+        mutt_buffer_len (src) &&
+        *mutt_b2s (src) != '/')
+    {
+      if (mutt_getcwd (tmp))
+      {
+        /* Note, mutt_pretty_mailbox() does '..' and '.' handling. */
+        if (mutt_buffer_len (tmp) > 1)
+          mutt_buffer_addch (tmp, '/');
+        mutt_buffer_addstr (tmp, mutt_b2s (src));
+        mutt_buffer_strcpy (src, mutt_b2s (tmp));
+      }
+    }
+
+  mutt_buffer_pool_release (&tmp);
+}
+
+void mutt_buffer_remove_path_password (BUFFER *dest, const char *src)
+{
+  mutt_buffer_clear (dest);
+  if (!(src && *src))
+    return;
+
+#ifdef USE_IMAP
+  if (mx_is_imap (src))
+    imap_buffer_remove_path_password (dest, src);
+  else
+#endif
+    mutt_buffer_strcpy (dest, src);
 }
 
 /* Extract the real name from /etc/passwd's GECOS field.
@@ -781,7 +804,7 @@ int mutt_needs_mailcap (BODY *m)
   return 1;
 }
 
-int mutt_is_text_part (BODY *b)
+int mutt_is_text_part (const BODY *b)
 {
   int t = b->type;
   char *s = b->subtype;
@@ -839,6 +862,7 @@ void mutt_free_envelope (ENVELOPE **p)
   rfc822_free_address (&(*p)->mail_followup_to);
 
   FREE (&(*p)->list_post);
+
   FREE (&(*p)->subject);
   /* real_subj is just an offset to subject and shouldn't be freed */
   FREE (&(*p)->disp_subj);
@@ -917,9 +941,12 @@ void mutt_merge_envelopes(ENVELOPE* base, ENVELOPE** extra)
 void _mutt_buffer_mktemp (BUFFER *buf, const char *prefix, const char *suffix,
                           const char *src, int line)
 {
-  mutt_buffer_printf (buf, "%s/%s-%s-%d-%d-%ld%ld%s%s",
+  RANDOM64 random64;
+  mutt_random_bytes(random64.char_array, sizeof(random64));
+
+  mutt_buffer_printf (buf, "%s/%s-%s-%d-%d-%"PRIu64"%s%s",
                       NONULL (Tempdir), NONULL (prefix), NONULL (Hostname),
-                      (int) getuid (), (int) getpid (), random (), random (),
+                      (int) getuid (), (int) getpid (), random64.int_64,
                       suffix ? "." : "", NONULL (suffix));
   dprint (3, (debugfile, "%s:%d: mutt_mktemp returns \"%s\".\n", src, line, mutt_b2s (buf)));
   if (unlink (mutt_b2s (buf)) && errno != ENOENT)
@@ -930,9 +957,12 @@ void _mutt_buffer_mktemp (BUFFER *buf, const char *prefix, const char *suffix,
 void _mutt_mktemp (char *s, size_t slen, const char *prefix, const char *suffix,
                    const char *src, int line)
 {
-  size_t n = snprintf (s, slen, "%s/%s-%s-%d-%d-%ld%ld%s%s",
+  RANDOM64 random64;
+  mutt_random_bytes(random64.char_array, sizeof(random64));
+
+  size_t n = snprintf (s, slen, "%s/%s-%s-%d-%d-%"PRIu64"%s%s",
                        NONULL (Tempdir), NONULL (prefix), NONULL (Hostname),
-                       (int) getuid (), (int) getpid (), random (), random (),
+                       (int) getuid (), (int) getpid (), random64.int_64,
                        suffix ? "." : "", NONULL (suffix));
   if (n >= slen)
     dprint (1, (debugfile, "%s:%d: ERROR: insufficient buffer space to hold temporary filename! slen=%zu but need %zu\n",
@@ -1050,11 +1080,44 @@ void mutt_pretty_mailbox (char *s, size_t buflen)
     *s++ = '=';
     memmove (s, s + len, mutt_strlen (s + len) + 1);
   }
-  else if (mutt_strncmp (s, Homedir, (len = mutt_strlen (Homedir))) == 0 &&
-	   s[len] == '/')
+  else
   {
-    *s++ = '~';
-    memmove (s, s + len - 1, mutt_strlen (s + len - 1) + 1);
+    BUFFER *cwd = mutt_buffer_pool_get ();
+    BUFFER *home = mutt_buffer_pool_get ();
+    int under_cwd = 0, under_home = 0, cwd_under_home = 0;
+    size_t cwd_len, home_len;
+
+    mutt_getcwd (cwd);
+    if (mutt_buffer_len (cwd) > 1)
+      mutt_buffer_addch (cwd, '/');
+    cwd_len = mutt_buffer_len (cwd);
+
+    mutt_buffer_strcpy (home, NONULL (Homedir));
+    if (mutt_buffer_len (home) > 1)
+      mutt_buffer_addch (home, '/');
+    home_len = mutt_buffer_len (home);
+
+    if (cwd_len &&
+        mutt_strncmp (s, mutt_b2s (cwd), cwd_len) == 0)
+      under_cwd = 1;
+
+    if (home_len &&
+        mutt_strncmp (s, mutt_b2s (home), home_len) == 0)
+      under_home = 1;
+
+    if (under_cwd && under_home && (home_len < cwd_len))
+      cwd_under_home = 1;
+
+    if (under_cwd && (!under_home || cwd_under_home))
+      memmove (s, s + cwd_len, mutt_strlen (s + cwd_len) + 1);
+    else if (under_home && (home_len > 1))
+    {
+      *s++ = '~';
+      memmove (s, s + home_len - 2, mutt_strlen (s + home_len - 2) + 1);
+    }
+
+    mutt_buffer_pool_release (&cwd);
+    mutt_buffer_pool_release (&home);
   }
 }
 
@@ -1121,67 +1184,28 @@ void _mutt_buffer_quote_filename (BUFFER *d, const char *f, int add_outer)
     mutt_buffer_addch (d, '\'');
 }
 
-static const char safe_chars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+@{}._-:%/";
+static const char safe_chars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+@{}._-:%";
 
-void mutt_buffer_sanitize_filename (BUFFER *d, const char *f, short slash)
+void mutt_buffer_sanitize_filename (BUFFER *d, const char *f, int flags)
 {
+  int allow_slash, allow_8bit;
+
   mutt_buffer_clear (d);
 
   if (!f)
     return;
 
+  allow_slash = flags & MUTT_SANITIZE_ALLOW_SLASH;
+  allow_8bit = flags & MUTT_SANITIZE_ALLOW_8BIT;
+
   for (; *f; f++)
   {
-    if ((slash && *f == '/') || !strchr (safe_chars, *f))
-      mutt_buffer_addch (d, '_');
-    else
+    if ((allow_slash && *f == '/')  ||
+        (allow_8bit && (*f & 0x80)) ||
+        strchr (safe_chars, *f))
       mutt_buffer_addch (d, *f);
-  }
-}
-
-static int is_ansi (const char *buf)
-{
-  while (*buf && (isdigit(*buf) || *buf == ';'))
-    buf++;
-  return (*buf == 'm');
-}
-
-/* Removes ANSI and backspace formatting.
- *
- * This logic is pulled from the pager fill_buffer() function, for use
- * in stripping reply-quoted autoview output of ansi sequences.
- */
-void mutt_buffer_strip_formatting (BUFFER *dest, const char *src)
-{
-  const char *s = src;
-
-  mutt_buffer_clear (dest);
-
-  if (!s)
-    return;
-
-  while (*s)
-  {
-    if (*s == '\010' && (s > src))
-    {
-      if (*(s+1) == '_')	/* underline */
-        s += 2;
-      else if (*(s+1) && mutt_buffer_len (dest))	/* bold or overstrike */
-      {
-        dest->dptr--;
-        mutt_buffer_addch (dest, *(s+1));
-        s += 2;
-      }
-      else			/* ^H */
-        mutt_buffer_addch (dest, *s++);
-    }
-    else if (*s == '\033' && *(s+1) == '[' && is_ansi (s + 2))
-    {
-      while (*s++ != 'm')	/* skip ANSI sequence */
-        ;
-    }
     else
-      mutt_buffer_addch (dest, *s++);
+      mutt_buffer_addch (d, '_');
   }
 }
 
@@ -1405,32 +1429,25 @@ const char *mutt_getcwd (BUFFER *cwd)
   return retval;
 }
 
-/* Note this function uses a fixed size buffer of LONG_STRING and so
- * should only be used for visual modifications, such as disp_subj. */
-char *mutt_apply_replace (char *dbuf, size_t dlen, char *sbuf, REPLACE_LIST *rlist)
+char *mutt_apply_replace (char *d, size_t dlen, char *s, REPLACE_LIST *rlist)
 {
   REPLACE_LIST *l;
   static regmatch_t *pmatch = NULL;
   static int nmatch = 0;
-  static char twinbuf[2][LONG_STRING];
-  int switcher = 0;
+  BUFFER *srcbuf, *destbuf;
   char *p;
-  int i, n;
-  size_t cpysize, tlen;
-  char *src, *dst;
+  unsigned int n;
 
-  if (dbuf && dlen)
-    dbuf[0] = '\0';
+  if (d && dlen)
+    d[0] = '\0';
 
-  if (sbuf == NULL || *sbuf == '\0' || (dbuf && !dlen))
-    return dbuf;
+  if (s == NULL || *s == '\0' || (d && !dlen))
+    return d;
 
-  twinbuf[0][0] = '\0';
-  twinbuf[1][0] = '\0';
-  src = twinbuf[switcher];
-  dst = src;
+  srcbuf = mutt_buffer_pool_get ();
+  destbuf = mutt_buffer_pool_get ();
 
-  strfcpy(src, sbuf, LONG_STRING);
+  mutt_buffer_strcpy (srcbuf, s);
 
   for (l = rlist; l; l = l->next)
   {
@@ -1441,18 +1458,15 @@ char *mutt_apply_replace (char *dbuf, size_t dlen, char *sbuf, REPLACE_LIST *rli
       nmatch = l->nmatch;
     }
 
-    if (regexec (l->rx->rx, src, l->nmatch, pmatch, 0) == 0)
+    if (regexec (l->rx->rx, mutt_b2s (srcbuf), l->nmatch, pmatch, 0) == 0)
     {
-      tlen = 0;
-      switcher ^= 1;
-      dst = twinbuf[switcher];
+      dprint (5, (debugfile, "mutt_apply_replace: %s matches %s\n",
+                  mutt_b2s (srcbuf), l->rx->pattern));
 
-      dprint (5, (debugfile, "mutt_apply_replace: %s matches %s\n", src, l->rx->pattern));
-
-      /* Copy into other twinbuf with substitutions */
+      mutt_buffer_clear (destbuf);
       if (l->template)
       {
-        for (p = l->template; *p && (tlen < LONG_STRING - 1); )
+        for (p = l->template; *p; )
         {
 	  if (*p == '%')
 	  {
@@ -1460,41 +1474,41 @@ char *mutt_apply_replace (char *dbuf, size_t dlen, char *sbuf, REPLACE_LIST *rli
 	    if (*p == 'L')
 	    {
 	      p++;
-              cpysize = MIN (pmatch[0].rm_so, LONG_STRING - tlen - 1);
-	      strncpy(&dst[tlen], src, cpysize);
-	      tlen += cpysize;
+              mutt_buffer_addstr_n (destbuf, mutt_b2s (srcbuf), pmatch[0].rm_so);
 	    }
 	    else if (*p == 'R')
 	    {
 	      p++;
-              cpysize = MIN (strlen (src) - pmatch[0].rm_eo, LONG_STRING - tlen - 1);
-	      strncpy(&dst[tlen], &src[pmatch[0].rm_eo], cpysize);
-	      tlen += cpysize;
+              mutt_buffer_addstr (destbuf, srcbuf->data + pmatch[0].rm_eo);
 	    }
 	    else
 	    {
-	      n = strtoul(p, &p, 10);               /* get subst number */
+	      if (!mutt_atoui (p, &n, MUTT_ATOI_ALLOW_TRAILING) && (n < l->nmatch))
+                mutt_buffer_addstr_n (destbuf, srcbuf->data + pmatch[n].rm_so,
+                                      pmatch[n].rm_eo - pmatch[n].rm_so);
 	      while (isdigit((unsigned char)*p))    /* skip subst token */
                 ++p;
-	      for (i = pmatch[n].rm_so; (i < pmatch[n].rm_eo) && (tlen < LONG_STRING-1); i++)
-	        dst[tlen++] = src[i];
 	    }
 	  }
 	  else
-	    dst[tlen++] = *p++;
+            mutt_buffer_addch (destbuf, *p++);
         }
       }
-      dst[tlen] = '\0';
-      dprint (5, (debugfile, "mutt_apply_replace: subst %s\n", dst));
+
+      mutt_buffer_strcpy (srcbuf, mutt_b2s (destbuf));
+      dprint (5, (debugfile, "mutt_apply_replace: subst %s\n", mutt_b2s (destbuf)));
     }
-    src = dst;
   }
 
-  if (dbuf)
-    strfcpy(dbuf, dst, dlen);
+  if (d)
+    strfcpy (d, mutt_b2s (srcbuf), dlen);
   else
-    dbuf = safe_strdup(dst);
-  return dbuf;
+    d = safe_strdup (mutt_b2s (srcbuf));
+
+  mutt_buffer_pool_release (&srcbuf);
+  mutt_buffer_pool_release (&destbuf);
+
+  return d;
 }
 
 
@@ -1553,7 +1567,7 @@ void mutt_FormatString (char *dest,		/* output buffer */
       srcbuf = mutt_buffer_from (srccopy);
       /* note: we are resetting dptr and *reading* from the buffer, so we don't
        * want to use mutt_buffer_clear(). */
-      srcbuf->dptr = srcbuf->data;
+      mutt_buffer_rewind (srcbuf);
       word = mutt_buffer_new ();
       command = mutt_buffer_new ();
 
@@ -1566,7 +1580,7 @@ void mutt_FormatString (char *dest,		/* output buffer */
         dprint(3, (debugfile, "fmtpipe +++: %s\n", srcbuf->dptr));
         if (word->data)
           *word->data = '\0';
-        mutt_extract_token(word, srcbuf, 0);
+        mutt_extract_token(word, srcbuf, MUTT_TOKEN_NOLISP);
         dprint(3, (debugfile, "fmtpipe %2d: %s\n", i++, word->data));
         mutt_buffer_addch(command, '\'');
         mutt_FormatString(buf, sizeof(buf), 0, cols, word->data, callback, data,
@@ -1977,8 +1991,9 @@ int mutt_save_confirm (const char *s, struct stat *st)
     if (option (OPTCONFIRMAPPEND))
     {
       tmp = mutt_buffer_pool_get ();
-      mutt_buffer_printf (tmp, _("Append messages to %s?"), s);
-      if ((rc = mutt_yesorno (mutt_b2s (tmp), MUTT_YES)) == MUTT_NO)
+      mutt_buffer_printf (tmp, _("Append message(s) to %s?"), s);
+      if ((rc = mutt_query_boolean (OPTCONFIRMAPPEND,
+                                    mutt_b2s (tmp), MUTT_YES)) == MUTT_NO)
 	ret = 1;
       else if (rc == -1)
 	ret = -1;
@@ -2005,7 +2020,8 @@ int mutt_save_confirm (const char *s, struct stat *st)
       {
         tmp = mutt_buffer_pool_get ();
 	mutt_buffer_printf (tmp, _("Create %s?"), s);
-	if ((rc = mutt_yesorno (mutt_b2s (tmp), MUTT_YES)) == MUTT_NO)
+	if ((rc = mutt_query_boolean (OPTCONFIRMCREATE,
+                                      mutt_b2s (tmp), MUTT_YES)) == MUTT_NO)
 	  ret = 1;
 	else if (rc == -1)
 	  ret = -1;
@@ -2124,6 +2140,7 @@ time_t mutt_decrease_mtime (const char *f, struct stat *st)
   struct utimbuf utim;
   struct stat _st;
   time_t mtime;
+  int rc;
 
   if (!st)
   {
@@ -2137,7 +2154,12 @@ time_t mutt_decrease_mtime (const char *f, struct stat *st)
     mtime -= 1;
     utim.actime = mtime;
     utim.modtime = mtime;
-    utime (f, &utim);
+    do
+      rc = utime (f, &utim);
+    while (rc == -1 && errno == EINTR);
+
+    if (rc == -1)
+      return -1;
   }
 
   return mtime;
@@ -2388,6 +2410,28 @@ void mutt_encode_path (BUFFER *dest, const char *src)
   FREE (&p);
 }
 
+/* returns: 0 - successful conversion
+ *         -1 - error: invalid input
+ *         -2 - error: out of range
+ */
+int mutt_atolofft (const char *str, LOFF_T *dst, int flags)
+{
+  int rc;
+  long long res;
+  LOFF_T tmp;
+  LOFF_T *t = dst ? dst : &tmp;
+
+  *t = 0;
+
+  if ((rc = mutt_atoll (str, &res, flags)) < 0)
+    return rc;
+  if ((LOFF_T) res != res)
+    return -2;
+
+  *t = (LOFF_T) res;
+  return rc;
+}
+
 
 /************************************************************************
  * These functions are transplanted from lib.c, in order to modify them *
